1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
926
927
928
929
930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
1083
1084
1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
1149
1150
1151
1152
1153
1154
1155
1156
1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
1184
1185
1186
1187
1188
1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
1220
1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
1236
1237
1238
1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
1252
1253
1254
1255
1256
1257
1258
1259
1260
1261
1262
1263
1264
1265
1266
1267
1268
1269
1270
1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
1308
1309
1310
1311
1312
1313
1314
1315
1316
1317
1318
1319
1320
1321
1322
1323
1324
1325
1326
1327
1328
1329
1330
1331
1332
1333
1334
1335
1336
1337
1338
1339
1340
1341
1342
1343
1344
1345
1346
1347
1348
1349
1350
1351
1352
1353
1354
1355
1356
1357
1358
1359
1360
1361
1362
1363
1364
1365
1366
1367
1368
1369
1370
1371
1372
1373
1374
1375
1376
1377
1378
1379
1380
1381
1382
1383
1384
1385
1386
1387
1388
1389
1390
1391
1392
1393
1394
1395
1396
1397
1398
1399
1400
1401
1402
1403
1404
1405
1406
1407
1408
1409
1410
1411
1412
1413
1414
1415
1416
1417
1418
1419
1420
1421
1422
1423
1424
1425
1426
1427
1428
1429
1430
1431
1432
1433
1434
1435
1436
1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
1447
1448
1449
1450
1451
1452
1453
1454
1455
1456
1457
1458
1459
1460
1461
1462
1463
1464
1465
1466
1467
1468
1469
1470
1471
1472
1473
1474
1475
1476
1477
1478
1479
1480
1481
1482
1483
1484
1485
1486
1487
1488
1489
1490
1491
1492
1493
1494
1495
1496
1497
1498
1499
1500
1501
1502
1503
1504
1505
1506
1507
1508
1509
1510
1511
1512
1513
1514
1515
1516
1517
1518
1519
1520
1521
1522
1523
1524
1525
1526
1527
1528
1529
1530
1531
1532
1533
1534
1535
1536
1537
1538
1539
1540
1541
1542
1543
1544
1545
1546
1547
1548
1549
1550
1551
1552
1553
1554
1555
1556
1557
1558
1559
1560
1561
1562
1563
1564
1565
1566
1567
1568
1569
1570
1571
1572
1573
1574
1575
1576
1577
1578
1579
1580
1581
1582
1583
1584
1585
1586
1587
1588
1589
1590
1591
1592
1593
1594
1595
1596
1597
1598
1599
1600
1601
1602
1603
1604
1605
1606
1607
1608
1609
1610
1611
1612
1613
1614
1615
1616
1617
1618
1619
1620
1621
1622
1623
1624
1625
1626
1627
1628
1629
1630
1631
1632
1633
1634
1635
1636
1637
1638
1639
1640
1641
1642
1643
1644
1645
1646
1647
1648
1649
1650
1651
1652
1653
1654
1655
1656
1657
1658
1659
1660
1661
1662
1663
1664
1665
1666
1667
1668
1669
1670
1671
1672
1673
1674
1675
1676
1677
1678
1679
1680
1681
1682
1683
1684
1685
1686
1687
1688
1689
1690
1691
1692
1693
1694
1695
1696
1697
1698
1699
1700
1701
1702
1703
1704
1705
1706
1707
1708
1709
1710
1711
1712
1713
1714
1715
1716
1717
1718
1719
1720
1721
1722
1723
1724
1725
1726
1727
1728
1729
1730
1731
1732
1733
1734
1735
1736
1737
1738
1739
1740
1741
1742
1743
1744
1745
1746
1747
1748
1749
1750
1751
1752
1753
1754
1755
1756
1757
1758
1759
1760
1761
1762
1763
1764
1765
1766
1767
1768
1769
1770
1771
1772
1773
1774
1775
1776
1777
1778
1779
1780
1781
1782
1783
1784
1785
1786
1787
1788
1789
1790
1791
1792
1793
1794
1795
1796
1797
1798
1799
1800
1801
1802
1803
1804
1805
1806
1807
1808
1809
1810
1811
1812
1813
1814
1815
1816
1817
1818
1819
1820
1821
1822
1823
1824
1825
1826
1827
1828
1829
1830
1831
1832
1833
1834
1835
1836
1837
1838
1839
1840
1841
1842
1843
1844
1845
1846
1847
1848
1849
1850
1851
1852
1853
1854
1855
1856
1857
1858
1859
1860
1861
1862
1863
1864
1865
1866
1867
1868
1869
1870
1871
1872
1873
1874
1875
1876
1877
1878
1879
1880
1881
1882
1883
1884
1885
1886
1887
1888
1889
1890
1891
1892
1893
1894
1895
1896
1897
1898
1899
1900
1901
1902
1903
1904
1905
1906
1907
1908
1909
1910
1911
1912
1913
1914
1915
1916
1917
1918
1919
1920
1921
1922
1923
1924
1925
1926
1927
1928
1929
1930
1931
1932
1933
1934
1935
1936
1937
1938
1939
1940
1941
1942
1943
1944
1945
1946
1947
1948
1949
1950
1951
1952
1953
1954
1955
1956
1957
1958
1959
1960
1961
1962
1963
1964
1965
1966
1967
1968
1969
1970
1971
1972
1973
1974
1975
1976
1977
1978
1979
1980
1981
1982
1983
1984
1985
1986
1987
1988
1989
1990
1991
1992
1993
1994
1995
1996
1997
1998
1999
2000
2001
2002
2003
2004
2005
2006
2007
2008
2009
2010
2011
2012
2013
2014
2015
2016
2017
2018
2019
2020
2021
2022
2023
2024
2025
2026
2027
2028
2029
2030
2031
2032
2033
2034
2035
2036
2037
2038
2039
2040
2041
2042
2043
2044
2045
2046
2047
2048
2049
2050
2051
2052
2053
2054
2055
2056
2057
2058
2059
2060
2061
2062
2063
2064
2065
2066
2067
2068
2069
2070
2071
2072
2073
2074
2075
2076
2077
2078
2079
2080
2081
2082
2083
2084
2085
2086
2087
2088
2089
2090
2091
2092
2093
2094
2095
2096
2097
2098
2099
2100
2101
2102
2103
2104
2105
2106
2107
2108
2109
2110
2111
2112
2113
2114
2115
2116
2117
2118
2119
2120
2121
2122
2123
2124
2125
2126
2127
2128
2129
2130
2131
2132
2133
2134
2135
2136
2137
2138
2139
2140
2141
2142
2143
2144
2145
2146
2147
2148
2149
2150
2151
2152
2153
2154
2155
2156
2157
2158
2159
2160
2161
2162
2163
2164
2165
2166
2167
2168
2169
2170
2171
2172
2173
2174
2175
2176
2177
2178
2179
2180
2181
2182
2183
2184
2185
2186
2187
2188
2189
2190
2191
2192
2193
2194
2195
2196
2197
2198
2199
2200
2201
2202
2203
2204
2205
2206
2207
2208
2209
2210
2211
2212
2213
2214
2215
2216
2217
2218
2219
2220
2221
2222
2223
2224
2225
2226
2227
2228
2229
2230
2231
2232
2233
2234
2235
2236
2237
2238
2239
2240
2241
2242
2243
2244
2245
2246
2247
2248
2249
2250
2251
2252
2253
2254
2255
2256
2257
2258
2259
2260
2261
2262
2263
2264
2265
2266
2267
2268
2269
2270
2271
2272
2273
2274
2275
2276
2277
2278
2279
2280
2281
2282
2283
2284
2285
2286
2287
2288
2289
2290
2291
2292
2293
2294
2295
2296
2297
2298
2299
2300
2301
2302
2303
2304
2305
2306
2307
2308
2309
2310
2311
2312
2313
2314
2315
2316
2317
2318
2319
2320
2321
2322
2323
2324
2325
2326
2327
2328
2329
2330
2331
2332
2333
2334
2335
2336
2337
2338
2339
2340
2341
2342
2343
2344
2345
2346
2347
2348
2349
2350
2351
2352
2353
2354
2355
2356
2357
2358
2359
2360
2361
2362
2363
2364
2365
2366
2367
2368
2369
2370
2371
2372
2373
2374
2375
2376
2377
2378
2379
2380
2381
2382
2383
2384
2385
2386
2387
2388
2389
2390
2391
2392
2393
2394
2395
2396
2397
2398
2399
2400
2401
2402
2403
2404
2405
2406
2407
2408
2409
2410
2411
2412
2413
2414
2415
2416
2417
2418
2419
2420
2421
2422
2423
2424
2425
2426
2427
2428
2429
2430
2431
2432
2433
2434
2435
2436
2437
2438
2439
2440
2441
2442
2443
2444
2445
2446
2447
2448
2449
2450
2451
2452
2453
2454
2455
2456
2457
2458
2459
2460
2461
2462
2463
2464
2465
|
# Tests for named references
%prep
# Required in order to declare an unset hash for substitution test
setopt TYPESET_TO_UNSET
: ${ZTST_continue::=1}
# The following test allows to assess what different types of named
# references refer to during their lifetime depending on where they
# were initialized and whether they were defined with or without the
# "-u" flag.
#
# The first parameter determines whether the named references are
# defined with or without the flag "-u".
#
# The second parameter determines where the named references are
# initialized. In all cases the named references are defined at the
# start of function "g". With value "0" they are initialized at the
# same place (in the same statement). With the other values, the
# initialization is delayed until later, the greater the value and the
# later the initialization.
function e() {
local s=$0 a=($0);
f "$@";
}
function f() {
local s=$0 a=($0);
g "$@";
}
function g() {
if (($2)); then local -n $1 rs ra;
else local -n $1 rs=s ra=a; fi;
if (($2 == 1)); then rs=s; ra=a; fi;
echo "$0:1: rs=$rs - ra=$ra";
local s=$0 a=($0);
if (($2 == 2)); then rs=s; ra=a; fi;
echo "$0:2: rs=$rs - ra=$ra";
h "$@";
echo "$0:3: rs=$rs - ra=$ra";
}
function h() {
if (($2 == 3)); then rs=s; ra=a; fi;
echo "$0:1: rs=$rs - ra=$ra";
local s=$0 a=($0);
if (($2 == 4)); then rs=s; ra=a; fi;
echo "$0:2: rs=$rs - ra=$ra";
i "$@";
echo "$0:3: rs=$rs - ra=$ra";
# Check that re-entering the same scope doesn't undo scope exit
k "$@"
}
function i() {
if (($2 == 5)); then rs=s; ra=a; fi;
echo "$0:1: rs=$rs - ra=$ra";
local s=$0 a=($0);
if (($2 == 6)); then rs=s; ra=a; fi;
echo "$0:2: rs=$rs - ra=$ra";
j "$@";
echo "$0:3: rs=$rs - ra=$ra";
k "$@";
}
function j() {
echo "$0:1: rs=$rs - ra=$ra";
local s=$0 a=($0);
echo "$0:2: rs=$rs - ra=$ra";
}
functions -c j k
local s=T a=(T);
function f1() {
typeset var=$0;
typeset -n ref1 ref2; typeset -n ref3=ref2;
function f2() {
typeset ref2=XX;
function f3() {
typeset var=$0;
function f4() {
typeset var=$0;
ref1=var;
ref3=var; # Initializes ref2 to var
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
f4;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
function g4() {
typeset var=$0;
function g5() {
typeset var=$0;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
g5;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
g4;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
f3;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
f2;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
function h2() {
typeset var=$0;
function h3() {
typeset var=$0;
function h4() {
typeset var=$0;
function h5() {
typeset var=$0;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
h5;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
h4;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
h3;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
h2;
echo "$0: ref1=$ref1 ref2=$ref2 ref3=$ref3";
}
# Helper function to test assigning and typesetting named references
# in different contexts.
#
# Runs the code passed in "$1" for all combinations of the following
# variables:
#
# - K: Kind of target reference. The assigned/typeset reference is
# one of the following kind:
# - "i": unInitialized reference:
# e.g., "ref0" in "typeset -n ref0"
# - "s": unSet reference:
# e.g., "ref0" in "typeset -n ref0; unset ref0"
# - "d": reference to not-yet-Defined variable:
# e.g., "ref0" in "typeset -n ref0=var" where "var" isn't defined
#
# - G: Global vs local references
# - "-g": All test variables/references are global ones.
# - "" : All test variables/references are local ones.
#
# - N: direct vs indirect references
# - "0": The target reference "ref0" is directly assigned/typeset.
# - "1": The target reference "ref0" is assigned/typeset via a
# named reference "ref1" that refers to "ref0".
test-setting-ref() {
local K G N;
for K in i s d; do
for G in -g ""; do
for N in 0 1; do
test-setting-ref-aux $1
done
done
done
unset -n ref0 ref1 var
}
# Helper function for the function test-setting-ref defined above.
#
# Runs the code passed in "$1" for the context specified by the
# variables "K", "G", and "N".
test-setting-ref-aux() {
unset -n ref0 ref1 var
# In several contexts different code paths are involved depending
# on whether the target reference "ref0" is assigned/typeset
# directly or via a named reference that refers to it. However, it
# doesn't seem to make a difference whether the chain of references
# is short or long. Therefore only chains of one reference are
# tested. One can manually double check that longer chains work the
# same by replacing the following line by one of the commented one
# just below and checking that all tests still pass.
((N < 1)) || typeset $G -n ref1=ref0
# ((N < 1)) || typeset $G -n refX=ref0 ref1=refX
# ((N < 1)) || typeset $G -n refX=ref0 refY=refX ref1=refY
local kind print
case $K in
i) kind="uninitialized reference" ; check=ref0; typeset $G -n ref0;;
s) kind="unset reference" ; check=ref0; typeset $G -n ref0; unset -n ref0;;
d) kind="reference to not-yet-defined"; check=var ; typeset $G -n ref0=var;;
*) echo "Unrecognized case: $case"; return;;
esac
echo "# $K:$kind - ${${G:-local}/-g/global} - ref$N"
{ eval ${(e)1}; typeset -p $check } always { TRY_BLOCK_ERROR=0 } 2>&1
}
%test
typeset -n ptr
typeset -n
0:minimal declaration
>ptr
typeset -n ptr=
typeset -n
0:nameref placeholder
>ptr=''
typeset -n ptr
ptr=var
typeset -n
0:assign nameref placeholder
>ptr=var
unset ptr
typeset -n ptr
typeset -n ptr=var
typeset -n
0:assign placeholder with new typeset
>ptr=var
typeset -n ptr1
typeset -n ptr2=ptr1
typeset -n
0:chain ending in placeholder
>ptr1
>ptr2=ptr1
typeset ptr=var
typeset -n ptr
typeset -n
0:convert scalar to nameref
>ptr=var
typeset -n ptr=var
typeset +n ptr
typeset -p ptr
0:remove nameref attribute
>typeset ptr=var
typeset -n ptr=gvar
() {
local ptr
typeset -p ptr
}
typeset -p ptr
0:Local non-reference hides outside reference
>typeset ptr
>typeset -n ptr=gvar
typeset -n ptr
typeset -p ptr
typeset -t ptr
typeset -p ptr
0:change type of a placeholder
F:Other type changes are fatal errors, should this also be?
>typeset -n ptr
>typeset -n ptr
*?*ptr: can't change type of a named reference
typeset -n ptr=var
typeset -t ptr
typeset -p ptr var
0:change type of referenced var
>typeset -n ptr=var
>typeset -t var
typeset var
unset var
typeset -n ptr=var
typeset -t ptr
typeset -p ptr var
0:change type of unset referenced var
F:regression - at one time this incorrectly applied the tag to "ptr"
F:note this causes "var" to become set
>typeset -n ptr=var
>typeset -t var
typeset -n ptr[1]=var
1:invalid nameref name
*?*reference variable cannot be an array
typeset var=value
typeset -n ptr=var
print $ptr
print $ptr[2,4]
0:nameref expansion, no braces
>value
>alu
typeset var=value
typeset -n ptr=var
print ${ptr}
print ${(U)ptr}
print ${ptr[2,5][1,3]}
print ${(U)ptr[2,5][1,3]}
0:nameref expansion, braces
>value
>VALUE
>alu
>ALU
typeset var=(val1 val2)
typeset -n ptr=var
print $ptr
print $ptr[2]
0:nameref array expansion, no braces
>val1 val2
>val2
typeset var=(val1 val2)
typeset -n ptr=var
print ${ptr}
print ${(O)ptr}
print ${ptr[2]}
print ${(U)ptr[2]}
print ${ptr[2][2,4]}
print ${(U)ptr[2][2,4]}
0:nameref array expansion, braces
>val1 val2
>val2 val1
>val2
>VAL2
>al2
>AL2
typeset -A var=(key1 val1 key2 val2)
typeset -n ptr=var
print $ptr
print $ptr[key2]
0:nameref hash expansion, no braces
>val1 val2
>val2
typeset -A var=(key1 val1 key2 val2)
typeset -n ptr=var
print ${ptr}
print ${(kv)ptr}
print ${ptr[key2]}
print ${(U)ptr[key2]}
print ${ptr[key2][2,4]}
print ${(U)ptr[key2][2,4]}
0:nameref hash expansion, braces
>val1 val2
>key1 val1 key2 val2
>val2
>VAL2
>al2
>AL2
typeset var=value
typeset -n ptr=var
ptr=newvalue
typeset -p var
ptr[3]=xt
typeset -p var
typeset ptr=newvalue
typeset -p var
typeset ptr[3]=xt
typeset -p var
0:assign/typeset existing scalar via nameref
>typeset var=newvalue
>typeset var=nextvalue
>typeset var=newvalue
>typeset var=nextvalue
typeset var=(val1 val2)
typeset -n ptr=var
ptr=(new1 new2 new3 new4 new5)
typeset -p var
ptr[3]=new6
typeset -p var
ptr[2,4]=(new7 new8)
typeset -p var
typeset ptr=(new1 new2 new3 new4 new5)
typeset -p var
typeset ptr[3]=new6
typeset -p var
typeset ptr[2,4]=(new7 new8)
typeset -p var
0:assign/typeset existing array via nameref
>typeset -a var=( new1 new2 new3 new4 new5 )
>typeset -a var=( new1 new2 new6 new4 new5 )
>typeset -a var=( new1 new7 new8 new5 )
>typeset -a var=( new1 new2 new3 new4 new5 )
>typeset -a var=( new1 new2 new6 new4 new5 )
>typeset -a var=( new1 new7 new8 new5 )
typeset -A var=(key1 val1)
typeset -n ptr=var
ptr=(key2 val2 key3 val3)
typeset -p var
ptr[key3]=VAL3
ptr[key4]=val4
typeset -p var
typeset ptr=(key2 val2 key3 val3)
typeset -p var
typeset ptr[key3]=VAL3
typeset ptr[key4]=val4
typeset -p var
0:assign/typeset existing array via nameref
>typeset -A var=( [key2]=val2 [key3]=val3 )
>typeset -A var=( [key2]=val2 [key3]=VAL3 [key4]=val4 )
>typeset -A var=( [key2]=val2 [key3]=val3 )
>typeset -A var=( [key2]=val2 [key3]=VAL3 [key4]=val4 )
() {
unset var
typeset -n ptr=var
ptr=value
typeset -p var
unset var
typeset ptr=value
typeset -p var
}
typeset -p var
1:assign/typeset new scalar via nameref
>typeset -g var=value
>typeset var=value
*?*no such variable: var
() {
unset var
typeset -n ptr=var
ptr=(val1 val2)
typeset -p var
unset var
typeset ptr=(val1 val2)
typeset -p var
}
typeset -p var
1:assign/typeset new array via nameref
>typeset -g -a var=( val1 val2 )
>typeset -a var=( val1 val2 )
*?*no such variable: var
typeset var=value
typeset -n ptr=var
unset "ptr[3]"
typeset -p var
unset "ptr[2,3]"
typeset -p var
unset ptr
typeset -p var
typeset var=(val1 val2 val3 val4 val5)
unset "ptr[3]"
typeset -p var
unset "ptr[2,4]"
typeset -p var
unset ptr
typeset -p var
typeset -A var=(key1 val1 key2 val2)
unset "ptr[key2]"
typeset -p var
unset ptr
typeset -p var
0:unset via nameref
>typeset var=vaue
>typeset var=ve
>typeset -a var=( val1 val2 '' val4 val5 )
>typeset -a var=( val1 '' val5 )
>typeset -A var=( [key1]=val1 )
typeset var=value
typeset -n ptr=var
unset -n ptr
typeset -p var ptr
0:unset of the nameref itself
>typeset var=value
typeset -p ptr ptr1 ptr2 var
1:check state of paramtab ONE
F:unexpected side-effects of previous tests
*?*no such variable: ptr
*?*no such variable: ptr1
*?*no such variable: ptr2
*?*no such variable: var
unset var
typeset -n ptr2=var
typeset -n ptr1=ptr2
typeset var=value
typeset -p ptr1 ptr2 var
print $ptr1
0:indirect nameref expansion
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset var=value
>value
typeset -p ptr1 ptr2 var
1:check state of paramtab TWO
F:unexpected side-effects of previous tests
*?*no such variable: ptr1
*?*no such variable: ptr2
*?*no such variable: var
typeset var
typeset -n ptr2=var
typeset -n ptr1=ptr2
typeset ptr1=newvalue
typeset -p ptr1 ptr2 var
0:typeset existing parameter indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset var=newvalue
typeset var=value
typeset -n ptr2=var
typeset -n ptr1=ptr2
unset ptr1
typeset -p ptr1 ptr2 var
0:unset parameter indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
typeset -n ptr2=var
typeset -n ptr1=ptr2
typeset ptr1=newvalue
typeset -p ptr1 ptr2 var
0:typeset new parameter indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset var=newvalue
unset var
typeset -n ptr2=var
typeset -n ptr1=ptr2
typeset var=value
typeset -p ptr1 ptr2 var
ptr1=newvalue
typeset -p ptr1 ptr2 var
0:assign new parameter indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset var=value
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset var=newvalue
typeset -p ptr1 ptr2 var
1:check state of paramtab THREE
F:unexpected side-effects of previous tests
*?*no such variable: ptr1
*?*no such variable: ptr2
*?*no such variable: var
typeset -a var
typeset -n ptr2=var
typeset -n ptr1=ptr2
typeset ptr1=(val1 val2)
typeset -p ptr1 ptr2 var
0:typeset existing array indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset -a var=( val1 val2 )
typeset -n ptr2=var
typeset -n ptr1=ptr2
typeset ptr1=(val1 val2)
typeset -p ptr1 ptr2 var
0:typeset new array indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset -a var=( val1 val2 )
typeset -p ptr1 ptr2
1:check state of paramtab FOUR
F:unexpected side-effects of previous tests
*?*no such variable: ptr1
*?*no such variable: ptr2
unset var
typeset -n ptr2=var
typeset -n ptr1=ptr2
ptr1=(val1 val2)
typeset -p ptr1 ptr2 var
0:assign new array indirectly
>typeset -n ptr1=ptr2
>typeset -n ptr2=var
>typeset -g -a var=( val1 val2 )
typeset -n ptr1=ptr2
typeset -n ptr2=ptr1
1:direct nameref loop not allowed
*?*invalid self reference
unset var
typeset -gn ptr1=var
typeset -p ptr1
0:global reference to unset var
>typeset -g -n ptr1=var
unset -n ptr1
typeset -gn ptr1
typeset -p ptr1
ptr1=ptr1
1:global direct reference
>typeset -g -n ptr1
*?*invalid self reference
typeset -n ptr1=ptr2
typeset -n ptr2=ptr3
typeset -n ptr3=ptr1
1:indirect nameref loop not allowed
*?*invalid self reference
typeset -n ptr1 ptr2
ptr1=ptr2
ptr2=ptr1
1:looping assignment not allowed
*?*invalid self reference
typeset ptr1=not-a-ref
() {
typeset -n ptr1=ptr1
() {
typeset -n ptr1=ptr1
echo $ptr1
}
}
0:regression: not a self reference (test 1)
>not-a-ref
typeset -n ptr1
typeset -n ptr2=ptr1
() {
typeset ptr2=no-a-ref
typeset -n ptr1=ptr2
}
0:regression: not a self reference (test 2)
typeset ptr2=foo
() {
typeset -n ptr1=ptr2
typeset -n ptr2=ptr1
}
0:regression: not a self reference (test 3)
unset -n ptr1
typeset -n ptr1='not[2]good'
1:invalid nameref
*?*invalid variable name: not\[2\]good
unset -n ptr
unset gval
typeset -n ptr=gval
gval=global
() { local gval=local; print $ptr; typeset -p ptr gval }
0:up-reference part 1
>global
>typeset -g -n ptr=gval
>typeset gval=local
typeset -p ptr ptr1 ptr2 val
1:check state of paramtab FIVE
F:unexpected side-effects of previous tests
*?*no such variable: ptr
*?*no such variable: ptr1
*?*no such variable: ptr2
*?*no such variable: val
unset gval
typeset -n ptr1=gval
typeset gval
() { typeset gval=local; ptr1=global }
typeset -p ptr1 gval
0:up-reference assignment part 1
F:All tests run inside a function, so "typeset gval" creates a local;
F:if that were omitted, ptr1= assignment would create a true global
F:and the output below would change to "typeset -g gval=global"
>typeset -n ptr1=gval
>typeset gval=global
typeset -p ptr ptr1 ptr2 val gval
1:check state of paramtab SIX
F:unexpected side-effects of previous tests
*?*no such variable: ptr
*?*no such variable: ptr1
*?*no such variable: ptr2
*?*no such variable: val
*?*no such variable: gval
typeset gval=global
() {
typeset -n ptr=gval
local gval=local
print $ptr
}
typeset -p ptr gval
1:up-reference part 2
>global
*?*no such variable: ptr
>typeset gval=global
typeset -n ptr=gval
() {
local lval=local
typeset -n ptr=lval
ptr=LOCAL
typeset -p lval gval ptr
}
typeset -p ptr
0:localized namerefs hide global namerefs
*?*no such variable: gval
>typeset lval=LOCAL
>typeset -n ptr=lval
>typeset -n ptr=gval
typeset -A var=(myself outside)
() {
typeset -n myself=var
local -h var
print -r -- $myself[myself]
typeset -p var
}
0:up-reference part 3, hidden global
>outside
>typeset -h var
() {
typeset notdef
unset notdef
() {
typeset -n ptr=notdef
ptr=(DEFINED)
}
typeset -p notdef
}
0:up-reference part 4, unset local and type change
>typeset -a notdef=( DEFINED )
() {
typeset -n ptr1=ptr2
typeset -n ptr2
typeset -p ptr1 ptr2
typeset val=LOCAL
() {
ptr1=val
typeset -n
printf "%s=%s\n" ptr1 "$ptr1" ptr2 "$ptr2"
}
typeset -p ptr1 ptr2
}
typeset -p ptr2
1:up-reference part 5, stacked namerefs, end not in scope
>typeset -n ptr1=ptr2
>typeset -n ptr2
>ptr1=ptr2
>ptr2=val
>ptr1=LOCAL
>ptr2=LOCAL
>typeset -n ptr1=ptr2
>typeset -n ptr2=val
*?*no such variable: ptr2
typeset ptr2
() {
typeset -n ptr1=ptr2
typeset -n ptr2
typeset -p ptr1 ptr2
typeset val=LOCAL
() {
ptr1=val
typeset -n
printf "%s=%s\n" ptr1 "$ptr1" ptr2 "$ptr2"
}
typeset -p ptr1 ptr2
}
typeset -p ptr2
0:up-reference part 6, stacked namerefs, end is in scope
F:Same test, should part 5 output look like this?
>typeset -n ptr1=ptr2
>typeset -n ptr2
>ptr1=ptr2
>ptr2
>ptr1=val
>ptr2=
>typeset -n ptr1=ptr2
>typeset -n ptr2
>typeset ptr2=val
() {
() {
local var
typeset -nu ptr1=var
ptr1=outer && print -u2 assignment expected to fail
typeset -n ptr2=var
ptr2=inner
typeset -n
printf "%s=%s\n" ptr1 "$ptr1" ptr2 "$ptr2"
}
typeset -p var
}
typeset -p var
1:up-reference part 7, upscope namerefs, end not in scope
>ptr1=var
>ptr2=var
>ptr1=
>ptr2=inner
*?*typeset*: no such variable: var
*?*typeset*: no such variable: var
typeset var
() {
() {
local var
typeset -nu ptr1=var
ptr1=outer || print -u2 assignment expected to succeed
typeset -n ptr2=var
ptr2=inner
typeset -n
printf "%s=%s\n" ptr1 "$ptr1" ptr2 "$ptr2"
}
typeset -p var
}
typeset -p var
0:up-reference part 8, upscope namerefs, end in scope
>ptr1=var
>ptr2=var
>ptr1=outer
>ptr2=inner
>typeset -g var=outer
>typeset var=outer
if zmodload zsh/parameter; then
() {
zmodload -u zsh/parameter
typeset -n myself=parameters
local -h parameters
print -r -- $myself[myself]
typeset -p parameters
}
else ZTST_skip='Cannot zmodload zsh/parameter, skipping autoload test'
fi
0:up-reference part 9, autoloading with hidden special
>nameref-local-association-readonly-hide-hideval-special
>typeset -h parameters
(
inner() { local -n var="${1:?}"; var=(alpha beta gamma); }
outer() { local -a foo=(outer); inner foo; typeset -p foo; }
foo=3 ; { outer foo } always { typeset -p foo }
)
0:up-reference part 10, assignment to enclosing scope, types match
>typeset -a foo=( alpha beta gamma )
>typeset -g foo=3
(
inner() { local -n var="${1:?}"; var=(alpha beta gamma); }
outer() { local foo=outer; inner foo; typeset -p foo; }
foo=3 ; { outer foo } always { typeset -p foo }
)
0:up-reference part 11, assignment to enclosing scope, type mismatch
>typeset -a foo=( alpha beta gamma )
>typeset -g foo=3
(
inner() { local -n var="${1:?}"; unset var; var=(alpha beta gamma); }
outer() { local foo=outer; inner foo; typeset -p foo; }
foo=3 ; { outer foo } always { typeset -p foo }
)
0:up-reference part 12, assignment to enclosing scope, unset by reference
>typeset -a foo=( alpha beta gamma )
>typeset -g foo=3
(
inner() { local "${1:?}"; local -nu var="$1"; var=(alpha beta gamma); }
outer() { local -a foo=(outer); inner foo; typeset -p foo; }
foo=3 ; { outer foo } always { typeset -p foo }
)
0:up-reference part 13, assignment to enclosing scope, skip local
>typeset -a foo=( alpha beta gamma )
>typeset -g foo=3
(
inner() { local "${1:?}"; local -nu var="$1";
typeset -g var=(alpha beta gamma); }
outer() { local -a foo=(outer); inner foo; typeset -p foo; }
foo=3 ; { outer foo } always { typeset -p foo }
)
0f:up-reference part 14, typeset -g to enclosing scope, skip local
F:typeset cannot bypass a name in the local scope, even via nameref
>typeset -a foo=( alpha beta gamma )
>typeset -g foo=3
() {
# scope with no parameters
() {
local -nu upref=$1
local var=at_upref
print -- $upref
} var
}
0:up-reference part 15, non-existent parameter in outer scope
# no output expected
>
if [[ $options[typesettounset] != on ]]; then
ZTST_skip='Ignoring zmodload bug that resets TYPESET_TO_UNSET'
setopt typesettounset
fi
0:options reloaded
F:Checking for a bug in zmodload that affects later tests
typeset ptr2=var2
typeset var2=GLOBAL
() {
typeset -n ptr1=ptr2
typeset ptr2=var1
typeset var1=VAR1
typeset var2=VAR2
print -r -- ${(P)ptr1}
}
0:Order of evaluation with ${(P)...}
>VAR2
ary=(one two three four)
typeset -n ptr=ary
print -r ${(j.:.)ptr//o/0}
0:expansion flags and string replacement
>0ne:tw0:three:f0ur
var=value
typeset -n ptr=var
myscalar=ptr
echo ${(P)myscalar}
0:named references with (P), as ${(P)name_of_nameref}
>value
var=value
myscalar=var
typeset -n ptr=myscalar
echo ${(P)ptr}
0:named references with (P), as ${(P)nameref}
>value
unset -n ref
unset var
typeset -n ref=var
typeset var=GLOBAL
() {
typeset -n ref=$1
print -r $ref
ref=RESET
typeset -p ref var
} ref
typeset -p ref var
0:local reference points to same-name global reference, part 1
>GLOBAL
>typeset -n ref=ref
>typeset -g var=RESET
>typeset -n ref=var
>typeset var=RESET
unset -n ref
unset var
typeset -n ref=var
() {
typeset -n ref=$1
print -r $ref
ref=RESET
typeset -p ref var
} ref
typeset -p ref var
0:local reference points to same-name global reference, part 2
>
>typeset -n ref=ref
>typeset -g var=RESET
>typeset -n ref=var
>typeset -g var=RESET
unset -n ref
unset var1
typeset -n ref=var1
() {
typeset -n ref=var2
ref=RESET
typeset -p ref var2
}
typeset -p ref var2
0:local reference hides same-name global reference
>typeset -n ref=var2
>typeset -g var2=RESET
>typeset -n ref=var1
>typeset -g var2=RESET
unset -n ref
unset one
typeset -n ref
typeset one=ONE
for ref in one ref two; do print -r $ref; done
1:for-loop variable is a reference, part 1
>ONE
*?*ref: invalid self reference
unset -n ref
unset one
typeset -n ref
() {
typeset one=ONE
for ref in one ref two; do print -r ${(t)ref}; done
}
1:for-loop variable is a reference, part 2
>scalar-local
*?*ref: invalid self reference
unset -n ref
unset one var
typeset -n ref=var
() {
typeset one=ONE
typeset -n ref=ref
for ref in one ref two; do
typeset -p ref
print -r $ref
done
typeset -p ref
}
typeset -p ref
0:for-loop variable is a reference, part 3
>typeset -n ref=one
>ONE
>typeset -n ref=ref
>
>typeset -n ref=two
>
>typeset -n ref=two
>typeset -n ref=var
typeset -g .K01.scalar='RW'
typeset -gA .K01.assoc=(x y)
typeset -ga .K01.array=(z)
typeset -gi .K01.integer=0
typeset -gE .K01.double=0.0
typeset -gF .K01.float=0.0
typeset -gr .K01.readonly='RO'
typeset -n gref
for gref in ARGC .K01.{scalar,assoc,array,integer,double,float,readonly}
do
{ unset gref } always { TRY_BLOCK_ERROR=0 }
done
typeset -p .K01.{scalar,assoc,array,integer,double,float,readonly}
unset .K01.{scalar,assoc,array,integer,double,float}
0:unset various types via nameref, including a readonly special
>typeset -g -r .K01.readonly=RO
*?*read-only variable: ARGC
*?*read-only variable: .K01.readonly
*?*no such variable: .K01.scalar
*?*no such variable: .K01.assoc
*?*no such variable: .K01.array
*?*no such variable: .K01.integer
*?*no such variable: .K01.double
*?*no such variable: .K01.float
unset -n ref
unset one
typeset -n ref
() {
setopt localoptions warn_nested_var
typeset one=ONE
for ref in one two; do print -r ${(t)ref}; done
typeset -n ref
for ref in one two; do print -r ${(t)ref}; done
}
0:for-loop variable is a reference, part 4, warnings
>scalar-local
>
>scalar-local
>
*?*reference ref*to local variable one
typeset -n ref=var
{
for ref in valid1 inv@lid valid2; do typeset -p ref; done
} always { TRY_BLOCK_ERROR=0 }
typeset -nr ref=var
{
for ref in valid3 valid4; do typeset -p ref; done
} always { TRY_BLOCK_ERROR=0 }
1:for-loop variable is a reference, part 5, errors
>typeset -n ref=valid1
*?*invalid variable name: inv@lid
*?*read-only reference: ref
unset -n ref
typeset -n ref
() {
setopt localoptions warn_nested_var
typeset inner
ref=inner
}
typeset -p ref
0:Global variable is a reference, warning
>typeset -n ref=inner
*?*reference ref*to local variable inner
unset -n ref
typeset -n ref=GLOBAL
() {
typeset -gn ref=RESET
}
typeset -p ref
0:reset global reference within function
>typeset -n ref=RESET
unset -n ref
typeset -rn ref=RO
typeset -p ref
(typeset -n ref=RW)
print status: $? expected: 1
typeset +r -n ref
typeset -p ref
typeset -r +n ref
typeset -p ref
(typeset -rn ref)
print status: $? expected: 1
typeset +r -n ref=RW # Assignment occurs after type change,
typeset -p ref RO # so RO=RW here. Potentially confusing.
typeset -r -n ref=RX # No type change, so referent changes ...
typeset -p ref RO # ... and previous refererent does not.
typeset +rn ref=RW # Here ref=RW, again type changed first.
typeset -p ref
0:add and remove readonly attribute with references
>typeset -rn ref=RO
*?*: ref: read-only reference
>status: 1 expected: 1
>typeset -n ref=RO
>typeset -r ref=RO
*?*: ref: read-only variable
>status: 1 expected: 1
>typeset -n ref=RO
>typeset -g RO=RW
>typeset -rn ref=RX
>typeset -g RO=RW
>typeset ref=RW
() {
typeset -n r1 r2=
typeset -p r1 r2
print -- ${(!)r1-unset}
print -- ${+r1}
typeset -p r1
}
0:unset nameref remains unset when resolved
F:relies on global TYPESET_TO_UNSET in %prep
>typeset -n r1
>typeset -n r2=''
>unset
>0
>typeset -n r1
bar=xx
typeset -n foo=bar
() {
typeset -n foo; foo=zz
foo=zz || print -u2 foo: assignment failed
typeset -p bar zz
}
# prior to workers/53676 the assignment failed
unset zz
() { typeset -n foo; foo=zz; local zz; foo=zz; typeset -p bar zz }
0:regression: local nameref may not in-scope a global parameter
F:previously this could create an infinite recursion and crash
>typeset -g bar=xx
>typeset -g zz=zz
>typeset -g bar=xx
>typeset zz=zz
typeset -nm foo=bar
1:create nameref by pattern match not allowed
*?*typeset:1: -m not allowed with -n
typeset -nariu ref=var
1:regression: warn about invalid nameref options but never about valid ones
*?*typeset:1: -a not allowed with -n
*?*typeset:1: -i not allowed with -n
e -u 0
0:assignment at different scope than declaration, -u 0
>g:1: rs=f - ra=f
>g:2: rs=f - ra=f
>h:1: rs=f - ra=f
>h:2: rs=f - ra=f
>i:1: rs=f - ra=f
>i:2: rs=f - ra=f
>j:1: rs=f - ra=f
>j:2: rs=f - ra=f
>i:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>h:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>g:3: rs=f - ra=f
e '' 0
0:assignment at different scope than declaration, '' 0
>g:1: rs=f - ra=f
>g:2: rs=f - ra=f
>h:1: rs=f - ra=f
>h:2: rs=f - ra=f
>i:1: rs=f - ra=f
>i:2: rs=f - ra=f
>j:1: rs=f - ra=f
>j:2: rs=f - ra=f
>i:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>h:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>g:3: rs=f - ra=f
e -u 2
0:assignment at different scope than declaration, -u 2
>g:1: rs= - ra=
>g:2: rs=f - ra=f
>h:1: rs=f - ra=f
>h:2: rs=f - ra=f
>i:1: rs=f - ra=f
>i:2: rs=f - ra=f
>j:1: rs=f - ra=f
>j:2: rs=f - ra=f
>i:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>h:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>g:3: rs=f - ra=f
e -u 6
0:assignment at different scope than declaration, -u 6
>g:1: rs= - ra=
>g:2: rs= - ra=
>h:1: rs= - ra=
>h:2: rs= - ra=
>i:1: rs= - ra=
>i:2: rs=f - ra=f
>j:1: rs=f - ra=f
>j:2: rs=f - ra=f
>i:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>h:3: rs=f - ra=f
>k:1: rs=f - ra=f
>k:2: rs=f - ra=f
>g:3: rs=f - ra=f
e '' 6
0:assignment at different scope than declaration, '' 6
>g:1: rs= - ra=
>g:2: rs= - ra=
>h:1: rs= - ra=
>h:2: rs= - ra=
>i:1: rs= - ra=
>i:2: rs=i - ra=i
>j:1: rs=i - ra=i
>j:2: rs=i - ra=i
>i:3: rs=i - ra=i
>k:1: rs=i - ra=i
>k:2: rs=i - ra=i
>h:3: rs=h - ra=h
>k:1: rs=h - ra=h
>k:2: rs=h - ra=h
>g:3: rs=g - ra=g
f1
0:Transitive references with scoping changes
>f4: ref1=f4 ref2=XX ref3=f4
>f3: ref1=f3 ref2=XX ref3=f3
>g5: ref1=f3 ref2=XX ref3=f3
>g4: ref1=f3 ref2=XX ref3=f3
>f3: ref1=f3 ref2=XX ref3=f3
>f2: ref1=f1 ref2=XX ref3=f1
>f1: ref1=f1 ref2=f1 ref3=f1
>h5: ref1=f1 ref2=f1 ref3=f1
>h4: ref1=f1 ref2=f1 ref3=f1
>h3: ref1=f1 ref2=f1 ref3=f1
>h2: ref1=f1 ref2=f1 ref3=f1
>f1: ref1=f1 ref2=f1 ref3=f1
() {
typeset -n ref1 ref2
local var=l1
() {
() {
local var=l3
ref1=var
echo A1: ref1=$ref1 ref2=$ref2
}
ref2=var
# At this point, "ref1" and "ref2" refer to the same variable
# "var". Going forward, they should behave the same.
echo A2: ref1=$ref1 ref2=$ref2
() {
local var=l3
echo A3: ref1=$ref1 ref2=$ref2
}
echo A4: ref1=$ref1 ref2=$ref2
local var=l2
echo A5: ref1=$ref1 ref2=$ref2
}
echo A6: ref1=$ref1 ref2=$ref2
}
() {
typeset -n ref1 ref2
() {
() {
local var=l3
ref1=var
echo B1: ref1=$ref1 ref2=$ref2
}
ref2=var
# At this point, "ref1" and "ref2" refer to the same undefined
# variable "var". Going forward, they should behave the same.
echo B2: ref1=$ref1 ref2=$ref2
() {
() {
local var=l4
echo B3: ref1=$ref1 ref2=$ref2
}
local var=l3
echo B4: ref1=$ref1 ref2=$ref2
}
local var=l2
echo B5: ref1=$ref1 ref2=$ref2
() {
() {
local var=l4
echo B6: ref1=$ref1 ref2=$ref2
}
local var=l3
echo B7: ref1=$ref1 ref2=$ref2
}
echo B8: ref1=$ref1 ref2=$ref2
}
local var=l1
echo B9: ref1=$ref1 ref2=$ref2
}
0:rebound nameref behaves the same as newly bound one
>A1: ref1=l3 ref2=
>A2: ref1=l1 ref2=l1
>A3: ref1=l1 ref2=l1
>A4: ref1=l1 ref2=l1
>A5: ref1=l1 ref2=l1
>A6: ref1=l1 ref2=l1
>B1: ref1=l3 ref2=
>B2: ref1= ref2=
>B3: ref1=l4 ref2=l4
>B4: ref1=l3 ref2=l3
>B5: ref1=l2 ref2=l2
>B6: ref1=l2 ref2=l2
>B7: ref1=l2 ref2=l2
>B8: ref1=l2 ref2=l2
>B9: ref1=l1 ref2=l1
check-valid-referent-name() {
local -n ref=$1
[[ $ref = $2 ]] || echo "Unexpected expansion result: ref=$1, \$ref=$ref != $2"
}
check-invalid-variable-name() {
{
local -- $1=value-$1 2>/dev/null
echo "Unexpected valid variable name: $1"
} always {
TRY_BLOCK_ERROR=0
}
}
check-invalid-referent-name() {
{
local -n -- ref=$1 2>/dev/null
echo "Unexpected valid referent name: $1"
} always {
TRY_BLOCK_ERROR=0
}
}
local name value;
# Test valid variable names.
for name in \
foo foo. foo.bar foo.456 foo.4ar \
.foo .foo.bar .foo.456
do
local $name=value-$name
check-valid-referent-name "$name" value-$name
check-invalid-referent-name "$name"@
check-invalid-referent-name "$name""[1]"
done
# Test valid builtin variable names
for name value in 123 "" \! 0 \? 0 \$ $$ - 569X _ -n; do
check-valid-referent-name "$name" $value
check-invalid-referent-name "$name"@
check-invalid-referent-name "$name""[1]"
done
# Test invalid variable names.
for name in \
.foo. .foo.4ar . foo.. \
123. 123.bar 123.456 123.4ar .. .foo.. \
.123 .123. .123.bar .123.456 .123.4ar \
1oo 1oo. 1oo.bar 1oo.456 1oo.4ar \
.1oo .1oo. .1oo.bar .1oo.456 .1oo.4ar
do
check-invalid-variable-name "$name"
check-invalid-referent-name "$name"
check-invalid-referent-name "$name"@
check-invalid-referent-name "$name""[1]"
done
# Any variable name with more than one non-leading '.' is invalid.
for name in {,.}{foo,123,1oo}.{bar,456,4ar}.{,baz,789,7az}; do
check-invalid-variable-name "$name"
check-invalid-referent-name "$name"
done
unfunction check-valid-referent-name
unfunction check-invalid-variable-name
unfunction check-invalid-referent-name
unset name value
0:references to invalid variable names
edgelocal() ( local -n x=$1; typeset -p x; print -r $x )
edgeupper() ( local -nu x=$1; typeset -p x; print -r $x )
for edge in argv ARGC \@ \* \# 0 00 2 02 \! \? \$ - _
do
edgelocal $edge foo
edgeupper $edge foo
done
unfunction edgelocal edgeupper
0q:references to builtin specials
>typeset -n x=argv
>argv foo
>typeset -n x=ARGC
>2
>typeset -n x=0
>edgelocal
>typeset -n x=00
>edgelocal
>typeset -n x=2
>foo
>typeset -n x=02
>foo
>typeset -n x=!
>0
>typeset -un x=!
>0
>typeset -n x='?'
>0
>typeset -un x='?'
>0
>typeset -n x='$'
>$$
>typeset -un x='$'
>$$
>typeset -n x=-
>569X
>typeset -un x=-
>569X
>typeset -n x=_
>x
>typeset -un x=_
>x
?edgeupper: invalid variable name: argv
?edgeupper: invalid variable name: ARGC
?edgelocal: invalid variable name: @
?edgeupper: invalid variable name: @
?edgelocal: invalid variable name: *
?edgeupper: invalid variable name: *
?edgelocal: invalid variable name: #
?edgeupper: invalid variable name: #
?edgeupper: invalid variable name: 0
?edgeupper: invalid variable name: 00
?edgeupper: invalid variable name: 2
?edgeupper: invalid variable name: 02
tst() {
typeset -n ref0=0 2>&1
typeset -n ref00=00 2>&1
typeset -n ref2=2 2>&1
typeset -n ref02=02 2>&1
echo ${(q)0} ${(q)*} - $ref0 - $ref00 - $ref2 - $ref02
ref0=TST
ref2=BBB
echo ${(q)0} ${(q)*} - $ref0 - $ref00 - $ref2 - $ref02
ref00=Tst
ref02=Bbb
echo ${(q)0} ${(q)*} - $ref0 - $ref00 - $ref2 - $ref02; typeset -p -- 0 00 2 02 2>&1
unset ref0
unset ref2
echo ${(q)0} ${(q)*} - $ref0 - $ref00 - $ref2 - $ref02; typeset -p -- 0 00 2 02 2>&1
ref0=tst
ref2=bbb
echo ${(q)0} ${(q)*} - $ref0 - $ref00 - $ref2 - $ref02; typeset -p -- 0 00 2 02 2>&1
{ ref0=(TST TST) 2>&1 } always { TRY_BLOCK_ERROR=0 }
{ ref2=(BBB BBB) 2>&1 } always { TRY_BLOCK_ERROR=0 }
echo ${(q)0} ${(q)*} - $ref0 - $ref00 - $ref2 - $ref02; typeset -p -- 0 00 2 02 2>&1
typeset -n ref5=5 2>&1
ref5=zzz
echo ${(q)0} ${(q)*}
unset ref2
echo ${(q)0} ${(q)*}
unset ref2
echo ${(q)0} ${(q)*}
unset ref2
echo ${(q)0} ${(q)*}
unset ref2
echo ${(q)0} ${(q)*}
unset ref2
echo ${(q)0} ${(q)*}
}
tst aaa bbb ccc
unfunction tst
0:references to positional parameters
>tst aaa bbb ccc - tst - tst - bbb - bbb
>TST aaa BBB ccc - TST - TST - BBB - BBB
>Tst aaa Bbb ccc - Tst - Tst - Bbb - Bbb
>tst:typeset:11: no such variable: 00
>tst:typeset:11: no such variable: 2
>tst:typeset:11: no such variable: 02
>typeset -g 0=Tst
>Tst aaa ccc - Tst - Tst - ccc - ccc
>tst:typeset:14: no such variable: 00
>tst:typeset:14: no such variable: 2
>tst:typeset:14: no such variable: 02
>typeset -g 0=Tst
>tst aaa bbb - tst - tst - bbb - bbb
>tst:typeset:17: no such variable: 00
>tst:typeset:17: no such variable: 2
>tst:typeset:17: no such variable: 02
>typeset -g 0=tst
>tst:18: 0: attempt to assign array value to non-array
>tst:19: 2: attempt to assign array value to non-array
>tst aaa bbb - tst - tst - bbb - bbb
>tst:typeset:20: no such variable: 00
>tst:typeset:20: no such variable: 2
>tst:typeset:20: no such variable: 02
>typeset -g 0=tst
>tst aaa bbb '' '' zzz
>tst aaa '' '' zzz
>tst aaa '' zzz
>tst aaa zzz
>tst aaa
>tst aaa
#
# The following tests are run in interactive mode, using PS3 as an
# assignable special with side-effects. This crashed at one time.
#
# Note bypassing TYPESET_TO_UNSET here
PS1= PS2= $ZTST_testdir/../Src/zsh -fis <<<'
typeset -n p=PS3
() {
typeset -p p
local p
typeset -p p
p=xx
typeset -p p
}
'
0:regression: assign to local that shadows global named reference
>typeset -g -n p=PS3
>typeset p=''
>typeset p=xx
# Note bypassing TYPESET_TO_UNSET here
PS1= PS2= $ZTST_testdir/../Src/zsh -fis <<<'
() {
typeset p=PS3
typeset -n p
p=zz
}
typeset -p PS3
'
0:regression - converting a string into a named reference
>typeset PS3=zz
# Testing named directories requires to run in interactive mode
# Use "${(kv)nameddirs}" because "typeset -p nameddirs" prints no value
PS1= $ZTST_testdir/../Src/zsh -fis <<<'
[[ -d Modules/zsh ]] && module_path=( $PWD/Modules )
typeset v1=/1 v2
typeset -n r1=v1 r2
{ echo "~v1:" ~v1 } 2>&1
{ echo "~r1:" ~r1 } 2>&1
echo "nameddirs=( ${(kv)nameddirs} )"
r1=/A
echo "nameddirs=( ${(kv)nameddirs} )"
hash -d v2=/2 r2=/R
echo "nameddirs=( ${(kv)nameddirs} )"
setopt autonamedirs
v2=V2
echo "nameddirs=( ${(kv)nameddirs} )"
r2=v2
echo "nameddirs=( ${(kv)nameddirs} )"
r2=/B
echo "nameddirs=( ${(kv)nameddirs} )"
'
0:Only string parameters are promoted to named directories
>~v1: /1
>zsh: no such user or named directory: r1
>nameddirs=( v1 /1 )
>nameddirs=( v1 /A )
>nameddirs=( v1 /A v2 /2 r2 /R )
>nameddirs=( v1 /A r2 /R )
>nameddirs=( v1 /A r2 /R )
>nameddirs=( v1 /A v2 /B r2 /R )
unset var1 var2
typeset -n ref1=var1
() {
typeset -n ref2=ref1
typeset -n ref1=var2
typeset -i ref2=42
typeset -p ref1 ref2 var1 var2
}
1:typeset reference chain to not-yet-defined variable including a hidden reference
?(anon):typeset:4: no such variable: var2
>typeset -n ref1=var2
>typeset -n ref2=ref1
>typeset -i var1=42
typeset -n ref1
typeset -n ref2=ref1
() {
typeset -n ref3=ref2
typeset ref2=foo
ref1=ref3
}
1:self reference chain including a hidden reference
?(anon):3: ref3: invalid self reference
zmodload -u zsh/random
echo z=${(M)${(f)${ zmodload -ap}}:#*SRANDOM*}
typeset -n ref=SRANDOM
echo z=${(M)${(f)${ zmodload -ap}}:#*SRANDOM*}
echo v=${ref/<->/integer}
zmodload -u zsh/random
echo z=${(M)${(f)${ zmodload -ap}}:#*SRANDOM*}
echo v=${ref/<->/integer}
0:Referring and dereferring an autoload variable loads it
>z=SRANDOM (zsh/random)
>z=
>v=integer
>z=SRANDOM (zsh/random)
>v=integer
typeset -n ref=SRANDOM
echo v=${ref/<->/integer}
zmodload -u zsh/random
echo z=${(M)${(f)${ zmodload -ap}}:#*SRANDOM*}
typeset -h SRANDOM=local-variable
echo v=${ref/<->/integer}
echo NOT REACHED
1:Dereferring an autoload variable fails to load it if its hidden
>v=integer
>z=SRANDOM (zsh/random)
?(eval):6: Can't add module parameter `SRANDOM': local parameter exists
?(eval):zsh/random:6: error when adding parameter `SRANDOM'
?(eval):6: autoloading module zsh/random failed to define parameter: SRANDOM
() {
typeset var1=var1
typeset var2=var2
typeset -n ref1=var1
echo "ref1=$ref1";
() {
typeset -n ref1=var2
typeset -n ref2=ref1
echo "ref1=$ref1";
echo "ref2=$ref2";
}
}
0:regression: don't follow references when computing base scope - part 1
>ref1=var1
>ref1=var2
>ref2=var2
() {
typeset var1=var1
typeset -n ref1=var1
echo ref1=$ref1;
() {
typeset var2=var2
typeset -n ref1
typeset -n ref2=ref1
ref1=var2
echo ref1=$ref1;
echo ref2=$ref2;
}
}
0:regression: don't follow references when computing base scope - part 2
>ref1=var1
>ref1=var2
>ref2=var2
typeset -n ref1
typeset -n ref2=ref1;
() {
typeset ref2=foo
ref1=ref2
echo reached
}
echo NOT REACHED
1:incidental reference loop triggers error
>reached
*?*: ref2: invalid self reference
typeset -A -g VAR0=(aa AA)
typeset -n -g REF0=VAR0
typeset -A -g var0=(aa AA)
typeset -A var1=(aa AA)
() {
typeset -A var2=(aa AA)
typeset -n ref0=var0 ref1=var1 ref2=var2
# Test initial association
typeset -pm VAR0 var\?
REF0=(zz ZZ); ref0=(zz ZZ); ref1=(zz ZZ); ref2=(zz ZZ);
typeset -pm VAR0 var\?
# Test change from association to string
REF0=foo; ref0=foo; ref1=foo; ref2=foo;
typeset -pm VAR0 var\?
REF0=bar; ref0=bar; ref1=bar; ref2=bar;
typeset -pm VAR0 var\?
# Test change from string to array
REF0=(aa AA); ref0=(aa AA); ref1=(aa AA); ref2=(aa AA);
typeset -pm VAR0 var\?
REF0=(zz ZZ); ref0=(zz ZZ); ref1=(zz ZZ); ref2=(zz ZZ);
typeset -pm VAR0 var\?
# Test change from array to string
REF0=foo; ref0=foo; ref1=foo; ref2=foo;
typeset -pm VAR0 var\?
}
0:type changes via assignments to references
>typeset -g -A VAR0=( [aa]=AA )
>typeset -g -A var0=( [aa]=AA )
>typeset -g -A var1=( [aa]=AA )
>typeset -A var2=( [aa]=AA )
>typeset -g -A VAR0=( [zz]=ZZ )
>typeset -g -A var0=( [zz]=ZZ )
>typeset -g -A var1=( [zz]=ZZ )
>typeset -A var2=( [zz]=ZZ )
>typeset -g VAR0=foo
>typeset -g var0=foo
>typeset -g var1=foo
>typeset var2=foo
>typeset -g VAR0=bar
>typeset -g var0=bar
>typeset -g var1=bar
>typeset var2=bar
>typeset -g -a VAR0=( aa AA )
>typeset -g -a var0=( aa AA )
>typeset -g -a var1=( aa AA )
>typeset -a var2=( aa AA )
>typeset -g -a VAR0=( zz ZZ )
>typeset -g -a var0=( zz ZZ )
>typeset -g -a var1=( zz ZZ )
>typeset -a var2=( zz ZZ )
>typeset -g VAR0=foo
>typeset -g var0=foo
>typeset -g var1=foo
>typeset var2=foo
typeset -A -g ass0=(aa AA)
typeset -A ass1=(aa AA)
typeset -a -g arr0=(aa AA)
typeset -a arr1=(aa AA)
typeset -g str0=foo
typeset str1=foo
() {
typeset -n Ass0=ass0 Ass1=ass1 Arr0=arr0 Arr1=arr1 Str0=str0 Str1=str1
typeset ass0 ass1 arr0 arr1 str0 str1
{ Ass0=foo } always { TRY_BLOCK_ERROR=0 }; echo $?
{ Ass1=foo } always { TRY_BLOCK_ERROR=0 }; echo $?
{ Arr0=foo } always { TRY_BLOCK_ERROR=0 }; echo $?
{ Arr1=foo } always { TRY_BLOCK_ERROR=0 }; echo $?
{ Str0=(x) } always { TRY_BLOCK_ERROR=0 }; echo $?
{ Str1=(x) } always { TRY_BLOCK_ERROR=0 }; echo $?
}
typeset -p ass0 ass1 arr0 arr1 str0 str1
0:can't change type of hidden variables via assignments to references
>1
>1
>1
>1
>1
>1
>typeset -g -A ass0=( [aa]=AA )
>typeset -A ass1=( [aa]=AA )
>typeset -g -a arr0=( aa AA )
>typeset -a arr1=( aa AA )
>typeset -g str0=foo
>typeset str1=foo
?(anon):3: can't change type of hidden variable: ass0
?(anon):4: can't change type of hidden variable: ass1
?(anon):5: can't change type of hidden variable: arr0
?(anon):6: can't change type of hidden variable: arr1
?(anon):7: can't change type of hidden variable: str0
?(anon):8: can't change type of hidden variable: str1
typeset -A ass=(aa AA)
typeset -a arr=(aa AA)
typeset -i int=42
typeset str=foo
typeset -n Ass=ass Arr=arr Int=int Str=str
{ typeset Ass=foo } always { TRY_BLOCK_ERROR=0 }; echo $?
{ typeset Arr=foo } always { TRY_BLOCK_ERROR=0 }; echo $?
{ typeset Int=(aa AA) } always { TRY_BLOCK_ERROR=0 }; echo $?
{ typeset Str=(aa AA) } always { TRY_BLOCK_ERROR=0 }; echo $?
typeset -p ass arr int str
0:type changes via plain typeset to references
F:converting from association/array to string should work here too
>1
>1
>0
>0
>typeset -A ass=( [aa]=AA )
>typeset -a arr=( aa AA )
>typeset -a int=( aa AA )
>typeset -a str=( aa AA )
?(eval):typeset:6: ass: inconsistent type for assignment
?(eval):typeset:7: arr: inconsistent type for assignment
typeset var
typeset -n ref=var
# Change type to string
unset var; typeset var=fubar; typeset ref=barfu; typeset -p var
unset var; typeset -i var=12345; typeset +i ref=barfu; typeset -p var
unset var; typeset -a var=(a A); typeset +a ref=barfu; typeset -p var
unset var; typeset -A var=(a A); typeset +A ref=barfu; typeset -p var
# Change type to integer
unset var; typeset var=fubar; typeset -i ref=56789; typeset -p var
unset var; typeset -i var=12345; typeset -i ref=56789; typeset -p var
unset var; typeset -a var=(a A); typeset -i ref=56789; typeset -p var
unset var; typeset -A var=(a A); typeset -i ref=56789; typeset -p var
# Change type to array
unset var; typeset var=fubar; typeset -a ref=(z Z); typeset -p var
unset var; typeset -i var=12345; typeset -a ref=(z Z); typeset -p var
unset var; typeset -a var=(a A); typeset -a ref=(z Z); typeset -p var
unset var; typeset -A var=(a A); typeset -a ref=(z Z); typeset -p var
# Change type to association
unset var; typeset var=fubar; typeset -A ref=(z Z); typeset -p var
unset var; typeset -i var=12345; typeset -A ref=(z Z); typeset -p var
unset var; typeset -a var=(a A); typeset -A ref=(z Z); typeset -p var
unset var; typeset -A var=(a A); typeset -A ref=(z Z); typeset -p var
0:type changes via typed typeset to references
>typeset var=barfu
>typeset var=barfu
>typeset var=barfu
>typeset var=barfu
>typeset -i var=56789
>typeset -i var=56789
>typeset -i var=56789
>typeset -i var=56789
>typeset -a var=( z Z )
>typeset -a var=( z Z )
>typeset -a var=( z Z )
>typeset -a var=( z Z )
>typeset -A var=( [z]=Z )
>typeset -A var=( [z]=Z )
>typeset -A var=( [z]=Z )
>typeset -A var=( [z]=Z )
typeset ptr1=foo
() {
typeset -n ptr1
typeset -n ptr2=ptr1
ptr1=ptr2
echo ptr1=$ptr1
}
1:regression: reference loop with same name enclosing variable
?(anon):3: ptr2: invalid self reference
typeset var=foo
typeset -n ref=var
() {
typeset -n ref
ref=""
ref=ref
echo $ref
}
0:reference always prefers enclosing variable over itself
>foo
typeset -n ref1
() {
typeset -n ref2=ref3
ref1=ref2
typeset -nu ref3=ref1
}
1:self reference via upper reference
?(anon):3: ref1: invalid self reference
# The call to "test-setting-ref 'ref$N=str'" expands to the
# equivalent of the following:
#
# # Test uninitialized references
# typeset -g -n ref0 ; ref0=str; typeset -p ref0; unset -n ref0
# typeset -g -n ref0 ref1=ref0; ref1=str; typeset -p ref0; unset -n ref0 ref1
# typeset -n ref0 ; ref0=str; typeset -p ref0; unset -n ref0
# typeset -n ref0 ref1=ref0; ref1=str; typeset -p ref0; unset -n ref0 ref1
# # Test unset references
# typeset -g -n ref0 ; unset -n ref0; ref0=str; typeset -p ref0; unset -n ref0
# ...
# # Test references to not-yet-defined variables
# typeset -g -n ref0=var ; ref0=str; typeset -p var ; unset -n ref0 var
# ...
test-setting-ref 'ref$N=str'
0:assign not fully defined reference with string
># i:uninitialized reference - global - ref0
>typeset -g -n ref0=str
># i:uninitialized reference - global - ref1
>typeset -g -n ref0=str
># i:uninitialized reference - local - ref0
>typeset -n ref0=str
># i:uninitialized reference - local - ref1
>typeset -n ref0=str
># s:unset reference - global - ref0
>typeset -g ref0=str
># s:unset reference - global - ref1
>typeset -g ref0=str
># s:unset reference - local - ref0
>typeset ref0=str
># s:unset reference - local - ref1
>typeset ref0=str
># d:reference to not-yet-defined - global - ref0
>typeset -g var=str
># d:reference to not-yet-defined - global - ref1
>typeset -g var=str
># d:reference to not-yet-defined - local - ref0
>typeset -g var=str
># d:reference to not-yet-defined - local - ref1
>typeset -g var=str
test-setting-ref 'ref$N=( foo bar )'
0:assign not fully defined reference with array
># i:uninitialized reference - global - ref0
>(eval):1: ref0: can't change type of a named reference
>typeset -g -n ref0
># i:uninitialized reference - global - ref1
>(eval):1: ref1: can't change type of a named reference
>typeset -g -n ref0
># i:uninitialized reference - local - ref0
>(eval):1: ref0: can't change type of a named reference
>typeset -n ref0
># i:uninitialized reference - local - ref1
>(eval):1: ref1: can't change type of a named reference
>typeset -n ref0
># s:unset reference - global - ref0
>typeset -g -a ref0=( foo bar )
># s:unset reference - global - ref1
>typeset -g -a ref0=( foo bar )
># s:unset reference - local - ref0
>typeset -a ref0=( foo bar )
># s:unset reference - local - ref1
>typeset -a ref0=( foo bar )
># d:reference to not-yet-defined - global - ref0
>typeset -g -a var=( foo bar )
># d:reference to not-yet-defined - global - ref1
>typeset -g -a var=( foo bar )
># d:reference to not-yet-defined - local - ref0
>typeset -g -a var=( foo bar )
># d:reference to not-yet-defined - local - ref1
>typeset -g -a var=( foo bar )
test-setting-ref 'typeset $G ref$N=str'
0:typeset not fully defined reference with string
># i:uninitialized reference - global - ref0
>typeset -g -n ref0=str
># i:uninitialized reference - global - ref1
>typeset -g -n ref0=str
># i:uninitialized reference - local - ref0
>typeset -n ref0=str
># i:uninitialized reference - local - ref1
>typeset -n ref0=str
># s:unset reference - global - ref0
>typeset -g ref0=str
># s:unset reference - global - ref1
>typeset -g ref0=str
># s:unset reference - local - ref0
>typeset ref0=str
># s:unset reference - local - ref1
>typeset ref0=str
># d:reference to not-yet-defined - global - ref0
>typeset -g var=str
># d:reference to not-yet-defined - global - ref1
>typeset -g var=str
># d:reference to not-yet-defined - local - ref0
>typeset var=str
># d:reference to not-yet-defined - local - ref1
>typeset var=str
test-setting-ref 'typeset $G -a ref$N=( foo bar )'
0:typeset not fully defined reference with array
># i:uninitialized reference - global - ref0
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -g -n ref0
># i:uninitialized reference - global - ref1
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -g -n ref0
># i:uninitialized reference - local - ref0
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -n ref0
># i:uninitialized reference - local - ref1
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -n ref0
># s:unset reference - global - ref0
>typeset -g -a ref0=( foo bar )
># s:unset reference - global - ref1
>typeset -g -a ref0=( foo bar )
># s:unset reference - local - ref0
>typeset -a ref0=( foo bar )
># s:unset reference - local - ref1
>typeset -a ref0=( foo bar )
># d:reference to not-yet-defined - global - ref0
>typeset -g -a var=( foo bar )
># d:reference to not-yet-defined - global - ref1
>typeset -g -a var=( foo bar )
># d:reference to not-yet-defined - local - ref0
>typeset -a var=( foo bar )
># d:reference to not-yet-defined - local - ref1
>typeset -a var=( foo bar )
typeset i42=42
test-setting-ref 'typeset $G -i ref$N=i42'
0:typeset not fully defined reference with integer
># i:uninitialized reference - global - ref0
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -g -n ref0
># i:uninitialized reference - global - ref1
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -g -n ref0
># i:uninitialized reference - local - ref0
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -n ref0
># i:uninitialized reference - local - ref1
>(eval):typeset:1: ref0: can't change type of a named reference
>typeset -n ref0
># s:unset reference - global - ref0
>typeset -g -i ref0=42
># s:unset reference - global - ref1
>typeset -g -i ref0=42
># s:unset reference - local - ref0
>typeset -i ref0=42
># s:unset reference - local - ref1
>typeset -i ref0=42
># d:reference to not-yet-defined - global - ref0
>typeset -g -i var=42
># d:reference to not-yet-defined - global - ref1
>typeset -g -i var=42
># d:reference to not-yet-defined - local - ref0
>typeset -i var=42
># d:reference to not-yet-defined - local - ref1
>typeset -i var=42
test-unset() {
typeset var0=foo
typeset -n ref1=var0 ref2=ref1
typeset cmd=(unset $@); echo "#" $cmd; $cmd
typeset -p var0 ref1 ref2
}
test-unset -n ref1
test-unset -n ref2
test-unset -n -m ref1
test-unset -n -m ref2
unfunction test-unset
0:unsetting references with -n unsets the references
># unset -n ref1
>typeset var0=foo
>typeset -n ref2=ref1
># unset -n ref2
>typeset var0=foo
>typeset -n ref1=var0
># unset -n -m ref1
>typeset var0=foo
>typeset -n ref2=ref1
># unset -n -m ref2
>typeset var0=foo
>typeset -n ref1=var0
test-unset() {
typeset var0=foo
typeset -n ref1=var0 ref2=ref1
typeset cmd=(unset $@); echo "#" $cmd; $cmd
typeset -p var0 ref1 ref2
}
test-unset ref1
test-unset ref2
test-unset -m ref1
test-unset -m ref2
unfunction test-unset
0:unsetting references without -n unsets the referred parameters
># unset ref1
>typeset -n ref1=var0
>typeset -n ref2=ref1
># unset ref2
>typeset -n ref1=var0
>typeset -n ref2=ref1
># unset -m ref1
>typeset -n ref1=var0
>typeset -n ref2=ref1
># unset -m ref2
>typeset -n ref1=var0
>typeset -n ref2=ref1
test-unset() {
typeset var0=12345
typeset -n ref1=var0 ref2=ref1
typeset cmd=(unset $@); echo "#" $cmd; $cmd
typeset -p var0
}
test-unset ref1"[3]"
test-unset ref2"[3]"
test-unset -n ref1"[3]"
test-unset -n ref2"[3]"
unfunction test-unset
0:unsetting subscripted references unsets the referred elements
># unset ref1[3]
>typeset var0=1245
># unset ref2[3]
>typeset var0=1245
># unset -n ref1[3]
>typeset var0=1245
># unset -n ref2[3]
>typeset var0=1245
test-unset() {
typeset -r var=foo
typeset -n ref=var
typeset cmd=(unset $@); echo "#" $cmd; { $cmd 2>&1 } always { TRY_BLOCK_ERROR=0 }
typeset -p var
}
test-unset var
test-unset -m var
test-unset ref
test-unset -m ref
test-unset var"[2]"
test-unset ref"[2]"
test-unset -n ref"[2]"
unfunction test-unset
0:unsetting read-only parameter triggers an error
># unset var
>test-unset:3: read-only variable: var
>typeset -r var=foo
># unset -m var
>test-unset:3: read-only variable: var
>typeset -r var=foo
># unset ref
>test-unset:3: read-only variable: var
>typeset -r var=foo
># unset -m ref
>test-unset:3: read-only variable: var
>typeset -r var=foo
># unset var[2]
>test-unset:3: read-only variable: var
>typeset -r var=foo
># unset ref[2]
>test-unset:3: read-only variable: var
>typeset -r var=foo
># unset -n ref[2]
>test-unset:3: read-only variable: var
>typeset -r var=foo
test-unset() {
typeset -n ref1 ref2=ref1
typeset cmd=(unset $@); echo "#" $cmd; $cmd
typeset -p ref1 ref2
}
test-unset ref1
test-unset ref2
test-unset -m ref1
test-unset -m ref2
unfunction test-unset
0:unsetting placeholder references or their referents has no effect
># unset ref1
>typeset -n ref1
>typeset -n ref2=ref1
># unset ref2
>typeset -n ref1
>typeset -n ref2=ref1
># unset -m ref1
>typeset -n ref1
>typeset -n ref2=ref1
># unset -m ref2
>typeset -n ref1
>typeset -n ref2=ref1
test-unset() {
typeset -n ref1=undefined ref2=ref1
typeset cmd=(unset $@); echo "#" $cmd; $cmd
typeset -p ref1 ref2
}
typeset -p undefined 2>&1
test-unset ref1
test-unset ref2
test-unset -m ref1
test-unset -m ref2
unfunction test-unset
0:unsetting references to not-yet-defined variables or their referents has no effect
>(eval):typeset:6: no such variable: undefined
># unset ref1
>typeset -n ref1=undefined
>typeset -n ref2=ref1
># unset ref2
>typeset -n ref1=undefined
>typeset -n ref2=ref1
># unset -m ref1
>typeset -n ref1=undefined
>typeset -n ref2=ref1
># unset -m ref2
>typeset -n ref1=undefined
>typeset -n ref2=ref1
test-unset() {
typeset -n refg1=g1 refl1=l1
() {
typeset -g g1=glb1 g2=glb2
typeset l1=lcl1 l2=lcl2
() {
typeset -n refg2=g2 refl2=l2
typeset cmd=(unset $@ refg1 refg2 refl1 refl2); echo "#" $cmd; $cmd
} $@
typeset -p g1 g2 l1 l2 2>&1
} $@
unset g1 g2
}
test-unset
test-unset -m
unfunction test-unset
0:unsetting references referring to parameters in enclosing scopes unsets the parameters
># unset refg1 refg2 refl1 refl2
>(anon):typeset:7: no such variable: g1
>(anon):typeset:7: no such variable: g2
># unset -m refg1 refg2 refl1 refl2
>(anon):typeset:7: no such variable: g1
>(anon):typeset:7: no such variable: g2
test-unset() {
typeset -g g=glb
typeset l=lcl
typeset -n refg=g refl=l
() {
typeset g=hide-g
typeset l=hide-l
typeset cmd=(unset $@ refg refl); echo "#" $cmd; $cmd
echo "# inner scope"
typeset -p g l 2>&1
} $@
echo "# outer scope"
typeset -p g l 2>&1
unset g
}
test-unset
test-unset -m
unfunction test-unset
0:unsetting references referring to hidden parameters unsets the hidden parameters
># unset refg refl
># inner scope
>typeset g=hide-g
>typeset l=hide-l
># outer scope
>test-unset:typeset:12: no such variable: g
># unset -m refg refl
># inner scope
>typeset g=hide-g
>typeset l=hide-l
># outer scope
>test-unset:typeset:12: no such variable: g
typeset -n ref1
typeset -n ref2
typeset -n ref3=ref2
typeset var=aaa
() {
typeset -i ref2=123 # Hides the reference ref2 in this scope and nested scopes
typeset var=bbb
() {
typeset var=ccc
ref1=var
ref3=var # From now on ref1 and ref3 should always refer to the same variable
echo A:ref1=$ref1 ref2=$ref2 ref3=$ref3
} # Both top-level references ref1 and ref2 should be rebound
echo B:ref1=$ref1 ref2=$ref2 ref3=$ref3
() {
typeset var=ddd # No reference should refer to this variable
echo C:ref1=$ref1 ref2=$ref2 ref3=$ref3
}
echo D:ref1=$ref1 ref2=$ref2 ref3=$ref3
} # Both top-level references ref1 and ref2 should be rebound
echo E:ref1=$ref1 ref2=$ref2 ref3=$ref3
() {
typeset var=eee # No reference should refer to this variable
echo F:ref1=$ref1 ref2=$ref2 ref3=$ref3
}
echo G:ref1=$ref1 ref2=$ref2 ref3=$ref3
0:hidden reference refers to a nested variable
>A:ref1=ccc ref2=123 ref3=ccc
>B:ref1=bbb ref2=123 ref3=bbb
>C:ref1=bbb ref2=123 ref3=bbb
>D:ref1=bbb ref2=123 ref3=bbb
>E:ref1=aaa ref2=aaa ref3=aaa
>F:ref1=aaa ref2=aaa ref3=aaa
>G:ref1=aaa ref2=aaa ref3=aaa
typeset ref
typeset var1=var1@scope1
typeset var2=var2@scope1
() { # enter scope 2
typeset var1=var1@scope2
typeset var2=var2@scope2
typeset -g -n ref=var1; echo A:$ref # ref added to scope 2
typeset -g -n ref=var2; echo B:$ref # ref added to scope 2
() { # enter scope 3
typeset var1=var1@scope3
typeset -g -n ref=var1; echo C:$ref # ref added to scope 3
() { # enter scope 4
typeset var1=var1@scope4
typeset var2=var2@scope4
typeset -g -n ref=var1; echo D:$ref # ref added to scope 4
typeset -g -n ref=var2; echo E:$ref # ref added to scope 4
} # leave scope 4: ref rebound to var2 in scope 2 and added to scope 2
echo F:$ref
} # leave scope 3: ref remains bound to var2 in scope 2
echo G:$ref
unset -n ref # ref is unset
echo H:$ref
} # leave scope 2: ref remains unset
echo I:$ref
0:reference refers successively to multiple variables in multiple nested scopes
>A:var1@scope2
>B:var2@scope2
>C:var1@scope3
>D:var1@scope4
>E:var2@scope4
>F:var2@scope2
>G:var2@scope2
>H:
>I:
typeset ref
() { # enter scope 2
typeset var=var
typeset -g -n ref=var; echo A:$ref # ref added to scope 2
unset -n ref
typeset -g -i16 ref=255; echo B:$ref # ref becomes an integer in base 16
} # leave scope 2: ref remains an integer in base 16
echo C:$ref
0:reference referring to a nested variable becomes an integer
>A:var
>B:16#FF
>C:16#FF
show() { printf "%-13s: %-3s - %-3s - %-3s\n" "$@" }
typeset var=aaa
typeset -n refU refN="" refS=var
show '${refX}' ${(q!)refU} ${(q!)refN} ${(q!)refS}
show '${+refX}' ${(q!)+refU} ${(q!)+refN} ${(q!)+refS}
show '${refX+1}' ${(q!)refU+1} ${(q!)refN+1} ${(q!)refS+1}
show '${refX:+1}' ${(q!)refU:+1} ${(q!)refN:+1} ${(q!)refS:+1}
echo
show '${refX-bbb}' ${(q!)refU-bbb} ${(q!)refN-bbb} ${(q!)refS-bbb}
show '${refX=bbb}' ${(q!)refU=bbb} ${(q!)refN=bbb} ${(q!)refS=bbb}
show '${refX}' ${(q!)refU} ${(q!)refN} ${(q!)refS}
show '${var}' ${var}
echo
typeset -n refU refN="" refS=var
show '${refX:-ccc}' ${(q!)refU:-ccc} ${(q!)refN:-ccc} ${(q!)refS:-ccc}
show '${refX:=ccc}' ${(q!)refU:=ccc} ${(q!)refN:=ccc} ${(q!)refS:=ccc}
show '${refX}' ${(q!)refU} ${(q!)refN} ${(q!)refS}
show '${var}' ${var}
echo
typeset -n refU refN="" refS=var
show '${refX::=ddd}' ${(q!)refU::=ddd} ${(q!)refN::=ddd} ${(q!)refS::=ddd}
show '${refX}' ${(q!)refU} ${(q!)refN} ${(q!)refS}
show '${var}' ${var}
unfunction show
0:reference itself is examined in substitutions with the ! flag
>${refX} : '' - '' - var
>${+refX} : 0 - 1 - 1
>${refX+1} : '' - 1 - 1
>${refX:+1} : '' - '' - 1
>
>${refX-bbb} : bbb - '' - var
>${refX=bbb} : bbb - '' - var
>${refX} : bbb - '' - var
>${var} : aaa - -
>
>${refX:-ccc} : ccc - ccc - var
>${refX:=ccc} : ccc - ccc - var
>${refX} : ccc - ccc - var
>${var} : aaa - -
>
>${refX::=ddd}: ddd - ddd - ddd
>${refX} : ddd - ddd - ddd
>${var} : aaa - -
typeset -n ref1
setopt allexport
ref1=var
typeset -n ref2
unsetopt allexport
typeset -p ref1 ref2
0:references are not exported
>typeset -n ref1=var
>typeset -n ref2
typeset -hn ref_h
typeset -Hn ref_H
0:unsusual but legal option combinations
typeset str0=abc
typeset -a arr0=(aa bb cc)
typeset -A hsh0=([0]=aa [1]=bb [2]=cc)
typeset -n str1=str0 arr1=arr0 hsh1=hsh0
typeset -n str2=str1 arr2=arr1 hsh2=hsh1
emulate ksh -c '
echo "${str0} - ${str1} - ${str2}"
echo "${str0[1]} - ${str1[1]} - ${str2[1]}"
echo "${str0[1,2]} - ${str1[1,2]} - ${str2[1,2]}"
echo "${str0[@]} - ${str1[@]} - ${str2[@]}"
echo "${arr0} - ${arr1} - ${arr2}"
echo "${arr0[1]} - ${arr1[1]} - ${arr2[1]}"
echo "${arr0[1,2]} - ${arr1[1,2]} - ${arr2[1,2]}"
echo "${arr0[@]} - ${arr1[@]} - ${arr2[@]}"
echo "${hsh0} - ${hsh1} - ${hsh2}"
echo "${hsh0[1]} - ${hsh1[1]} - ${hsh2[1]}"
echo "${hsh0[@]} - ${hsh1[@]} - ${hsh2[@]}"
echo
echo "${!str0} - ${!str1} - ${!str2}"
echo "${!str0[1]} - ${!str1[1]} - ${!str2[1]}"
echo "${!str0[1,2]} - ${!str1[1,2]} - ${!str2[1,2]}"
echo "${!str0[@]} - ${!str1[@]} - ${!str2[@]}"
echo "${!arr0} - ${!arr1} - ${!arr2}"
echo "${!arr0[1]} - ${!arr1[1]} - ${!arr2[1]}"
echo "$(exec 2>&1; echo ${!arr0[1,2]}) - $(exec 2>&1; echo ${!arr1[1,2]}) - $(exec 2>&1; echo ${!arr2[1,2]})"
echo "${!arr0[@]} - ${!arr1[@]} - ${!arr2[@]}"
echo "${!hsh0} - ${!hsh1} - ${!hsh2}"
echo "${!hsh0[1]} - ${!hsh1[1]} - ${!hsh2[1]}"
echo "${!hsh0[@]} - ${!hsh1[@]} - ${!hsh2[@]}"
'
0:emulation of ksh's bang always dereferences the expanded parameter
>abc - abc - abc
>b - b - b
>bc - bc - bc
>abc - abc - abc
>aa - aa - aa
>bb - bb - bb
>bb cc - bb cc - bb cc
>aa bb cc - aa bb cc - aa bb cc
>aa - aa - aa
>bb - bb - bb
>aa bb cc - aa bb cc - aa bb cc
>
>abc - abc - abc
>b - b - b
>bc - bc - bc
>abc - abc - abc
>aa - aa - aa
>1 - 1 - 1
>(eval):6: invalid subscript - (eval):6: invalid subscript - (eval):6: invalid subscript
>aa bb cc - aa bb cc - aa bb cc
>aa - aa - aa
>1 - 1 - 1
>0 1 2 - 0 1 2 - 0 1 2
%clean
|